/*
 * Single macro to perform interrupts (inserts an extra CALL)
 *
 * Example:
 *
 *		INTERRUPT(myint) func()
 *		{
 *			int local_variable;
 *			... code here ...
 *		}
 *
 * You must initialize the appropriate 8086 interrupt vector with
 * your code segment (CS) and the offset to the symbol 'myint'.
 *
 * When using this macro, your function can 'return' normally.
 *
 * This macro permits you to call 'func()' from within the mainline
 * program, as well as from the interrupt.
 *
 * This file **REQUIRES** the extended pre-processor (MCP).
 * (CC86 ... -P, or "Preprocess" step enabled in DDSIDE)
 */
#define	_SP_
#define	INTERRUPT(name) asm {\
name	PUSH	AX\
		PUSH	BX\
		PUSH	CX\
		PUSH	DX\
		PUSH	SI\
		PUSH	DI\
		PUSH	ES\
		PUSH	DS\
		PUSH	CS\
		POP		DS\
		MOV		AX,?temp\
		PUSH	AX\
		CALL	name+30\
		POP		AX\
		MOV		_SP_?temp,AX\
		POP		DS\
		POP		ES\
		POP		DI\
		POP		SI\
		POP		DX\
		POP		CX\
		POP		BX\
		POP		AX\
		IRET\
}
#undef _SP_
