*
* Common routine to process a format specifier, and build an
* output string with appropriate substitutions.
*
* BX	- Pointer to next argument
* DI	- Pointer to output string
*
_format_ MOV	SI,[BX]		Get format string
* Process next character from format string
?1	MOV	AL,[SI]		Get character from string
	INC	SI		Skip to next
	CMP	AL,#'%'		Format specifier?
	JZ	?2		Yes, handle it
* Non-format, copy to string
	MOV	[DI],AL		Write to output string
	INC	DI		Skip to next
	AND	AL,AL		End of string?
	JNZ	?1		No, keep going
	RET
* Test for '-' (Left justify)
?2	MOV	AL,[SI]		Get next char
	MOV	AH,#' '		Assume space
	CMP	AL,#'-'		Left justify?
	JNZ	?3		No, its OK
	OR	AH,#%10000000	Set LJ flag
	INC	SI		Skip '-'
	MOV	AL,[SI]		Get next char
* Test for leading '0' (zero fill)
?3	CMP	AL,#'0'		Zero fill?
	JNZ	?4		No, its OK
	OR	AH,#'0'		Convert to ZF
?4	XOR	CL,CL		Zero initial width
* Evaluate field width
?5	MOV	AL,[SI]		Get next char
	INC	SI		Skip to next
	SUB	AL,#'0'		Convert
	CMP	AL,#10		In range?
	JAE	?6		No, exit
	MOV	CH,CL		Save Copy
	ADD	CL,CL		CL *= 2
	ADD	CL,CL		CL *= 4
	ADD	CL,CH		CL *= 5
	ADD	CL,CL		CL *= 10
	ADD	CL,AL		Include this digit
	JMP	<?5		Get next digit
* Interpret the format specifier
?6	MOV	CH,AH		Save flags
	DEC	BX		Backup
	DEC	BX		Backup
	MOV	AL,-1[SI]	Get char back
	PUSH	SI		Save format
	MOV	SI,[BX]		SI = next argument
	PUSH	BX		Save argptr
* 'b' Binary number
	MOV	BL,#2		Base 2
	CMP	AL,#'b'		Binary output?
	JZ	?8		Yes, do it
* 'o' Octal number
	MOV	BL,#8		Base 8
	CMP	AL,#'o'		Octal output?
	JZ	?8		Yes, do it
* 'x' hexidecimal number
	MOV	BL,#16		Base 16
	CMP	AL,#'x'		Hex output?
	JZ	?8		Yes, do it
* 'u' unsigned number
	CMP	AL,#'u'		Unsigned number?
	JZ	?7		Yes, do it
* 'd' decimal number
	CMP	AL,#'d'		Decimal number?
	JNZ	?20		No, try next
	AND	SI,SI		Test sign
	JNS	?7		Its OK
	NEG	SI		Negate it
	MOV	<[DI],#'-'	Minus sign
	INC	DI		Skip to next
	AND	CL,CL		Field zero?
	JZ	?7		Don't update
	DEC	CL		Reduce field width
?7	MOV	BL,#10		Base 10 output
* Output number in SI using base in BL
?8	XOR	AX,AX		Get zero
	PUSH	AX		Save for later
	MOV	AX,SI		Get value
	XOR	BH,BH		Zero count
* Process value & stack digits (in reverse order)
?9	XOR	DX,DX		Zero top
	PUSH	BX		Save count
	XOR	BH,BH		Zero count
	DIV	BX		Get first digit
	POP	BX		Restore count
	INC	BH		Advance count
	CMP	DL,#10		In range?
	JB	?10		Its OK
	ADD	DL,#7		Adjust for ASCII
?10	ADD	DL,#'0'		Convert to ASCII
	PUSH	DX		Save remainder
	AND	AX,AX		More data
	JNZ	?9		Do them all
* Unstack digits into output buffer
	CALL	?40		Left fill if necessary
?11	POP	AX		Get char
	AND	AL,AL		End of line?
	JZ	?12		Yes, quit
	MOV	[DI],AL		Write to output
	INC	DI		Skip to next
	JMP	<?11		Keep going
?12	AND	CH,CH		Test RJ bit
	JNS	?13		No right justify
	CALL	?50		Right fill
?13	POP	BX		Restore ARGPTR
	POP	SI		Restore Format string
	JMP	?1		And get next
* 's' string
?20	CMP	AL,#'s'		String output?
	JNZ	?30		No, try next
	PUSH	SI		Save for later
	MOV	BH,#-1		Initial count
?21	INC	BH		Advance count
	MOV	AL,[SI]		Test and advance
	INC	SI		Skip to next
	AND	AL,AL		End of string?
	JNZ	?21		Find end
	POP	SI		Restore pointer
	CALL	?40		Left fill if necessary
?22	MOV	AL,[SI]		Get char
	INC	SI		Skip to next
	AND	AL,AL		At end?
	JZ	?12		Yes, quit
	MOV	[DI],AX		Write to output
	INC	DI		Skip to next
	JMP	<?22		And proceed
*'c' Character
?30	CMP	AL,#'c'		Character output?
	JZ	?31		Yes, use char
	MOV	SI,AX		Copy value to X
?31	MOV	BH,#1		Width=1
	CALL	?40		Left fill if necessary
	MOV	AX,SI		Get char
	MOV	[DI],AL		Write char
	INC	DI		Skip to next
	JMP	<?12		And proceed
*
* Pad the field assuming width in A
*
* DI=Output BH=width, CX=flag:fwidth
?40	AND	CH,CH		Test for LJ
	JNS	?50		Handle it
?41	RET
?50	AND	CL,CL		Fixed field?
	JZ	?41		No, exit
	MOV	AH,BH		Get actual width
	MOV	AL,CH		Get flags
	AND	AL,#%00111111	Get fill char
?52	CMP	AH,CL		At field width?
	JAE	?41		Yes, exit
	MOV	[DI],AL		Write character
	INC	DI		Skip to next
	INC	AH		Advance width
	JMP	<?52		And continue
